@@ -29,13 +29,16 @@
 #endif
 #include <stdio.h>
 #include <sys/types.h>
-#include "regex.h"
+#include "rx.h"
 #include "getopt.h"
 #if defined(STDC_HEADERS)
 #include <stdlib.h>
 #endif
 #if HAVE_STRING_H || defined(STDC_HEADERS)
 #include <string.h>
+#ifndef bzero
+#define bzero(s, n)	memset ((s), 0, (n))
+#endif
 #if !defined(STDC_HEADERS)
 #include <memory.h>
 #endif
@@ -43,6 +46,10 @@
 #include <strings.h>
 #endif
 
+#ifdef RX_MEMDBUG
+#include <malloc.h>
+#endif
+
 #ifndef HAVE_BCOPY
 #ifdef HAVE_MEMCPY
 #define bcopy(FROM,TO,LEN)  memcpy(TO,FROM,LEN)
@@ -68,7 +75,7 @@ bcopy (from, to, len)
 #endif
 #endif
 
-char *version_string = "GNU sed version 1.18";
+char *version_string = "GNU sed version 2.00";
 
 /* Struct vector is used to describe a chunk of a compiled sed program.  
  * There is one vector for the main program, and one for each { } pair,
@@ -247,7 +254,9 @@ void bad_prog P_ ((char *why));
 int inchar P_ ((void));
 void savchar P_ ((int ch));
 int compile_address P_ ((struct addr * addr));
-void compile_regex P_ ((int slash));
+char * last_regex_string = 0;
+void buffer_regex  P_ ((int slash));
+void compile_regex P_ ((void));
 struct sed_label *setup_jump P_ ((struct sed_label * list, struct sed_cmd * cmd, struct vector * vec));
 FILE *compile_filename P_ ((int readit));
 void read_file P_ ((char *name));
@@ -372,6 +381,7 @@ main (argc, argv)
 
   /* see regex.h */
   re_set_syntax (RE_SYNTAX_POSIX_BASIC);
+  rx_cache_bound = 10000;	/* Consume memory rampantly. */
 
   myname = argv[0];
   while ((opt = getopt_long (argc, argv, "hne:f:V", longopts, (int *) 0))
@@ -515,7 +525,11 @@ compile_file (str)
       if (ch == 'n')
 	no_default_output++;
       while (ch != EOF && ch != '\n')
-	ch = getc (prog_file);
+	{
+	  ch = getc (prog_file);
+	  if (ch == '\\')
+	    ch = getc (prog_file);
+	}
       ++prog_line;
     }
   else if (ch != EOF)
@@ -612,7 +626,11 @@ compile_program (vector, open_line)
 	  if (cur_cmd->a1.addr_type != 0)
 	    bad_prog (NO_ADDR);
 	  do
-	    ch = inchar ();
+	    {
+	      ch = inchar ();
+	      if (ch == '\\')
+		ch = inchar ();
+	    }
 	  while (ch != EOF && ch != '\n');
 	  vector->v_length--;
 	  goto skip_comment;
@@ -728,7 +746,8 @@ compile_program (vector, open_line)
 	case 's':
 	  cur_cmd->cmd = ch;
 	  slash = inchar ();
-	  compile_regex (slash);
+	  buffer_regex (slash);
+	  compile_regex ();
 
 	  cur_cmd->x.cmd_regex.regx = last_regex;
 
@@ -953,7 +972,8 @@ compile_address (addr)
       addr->addr_type = addr_is_regex;
       if (ch == '\\')
 	ch = inchar ();
-      compile_regex (ch);
+      buffer_regex (ch);
+      compile_regex ();
       addr->addr_regex = last_regex;
       do
 	ch = inchar ();
@@ -976,7 +996,7 @@ compile_address (addr)
 }
 
 void
-compile_regex (slash)
+buffer_regex (slash)
      int slash;
 {
   VOID *b;
@@ -1068,19 +1088,31 @@ compile_regex (slash)
     bad_prog (BAD_EOF);
   if (size_buffer (b))
     {
-      last_regex = (struct re_pattern_buffer *) ck_malloc (sizeof (struct re_pattern_buffer));
-      last_regex->allocated = size_buffer (b) + 10;
-      last_regex->buffer =
-	(unsigned char *) ck_malloc (last_regex->allocated);
-      last_regex->fastmap = ck_malloc (256);
-      last_regex->translate = 0;
-      re_compile_pattern (get_buffer (b), size_buffer (b), last_regex);
+      if (last_regex_string)
+	last_regex_string = (char *)ck_realloc (last_regex_string,
+						size_buffer (b) + 1);
+      else
+	last_regex_string = (char *)ck_malloc (size_buffer (b) + 1);
+      bcopy (get_buffer (b), last_regex_string, size_buffer (b));
+      last_regex_string [size_buffer (b)] = 0;
     }
   else if (!last_regex)
     bad_prog (NO_REGEX);
   flush_buffer (b);
 }
 
+void
+compile_regex ()
+{
+  
+  last_regex = ((struct re_pattern_buffer *)
+		ck_malloc (sizeof (struct re_pattern_buffer)));
+  bzero (last_regex, sizeof (*last_regex));
+  last_regex->fastmap = ck_malloc (256);
+  re_compile_pattern (last_regex_string,
+		      strlen (last_regex_string), last_regex);
+}
+
 /* Store a label (or label reference) created by a ':', 'b', or 't'
    comand so that the jump to/from the lable can be backpatched after
    compilation is complete */
@@ -1190,6 +1222,7 @@ read_file (name)
 	  return;
 	}
     }
+  
   while (read_pattern_space ())
     {
       execute_program (the_program);
@@ -1665,7 +1698,9 @@ restart:
 	  {
 	    unsigned char *p, *e;
 
-	    for (p = (unsigned char *) (line.text), e = p + line.length; p < e; p++)
+	    for (p = (unsigned char *) (line.text), e = p + line.length;
+		 p < e;
+		 p++)
 	      *p = cur_cmd->x.translate[*p];
 	  }
 	  break;
